################################################################################
###                                                                          ###
### Conversion from DMS to Decimal Degrees                                   ###
###                                                                          ###
### 1. Extract whole degrees                                                 ###
### 2. Calculate the total number of seconds min*60 + secs                   ###
### 3. The fractional part is total number of seconds divided by 3600.       ### 
### 4. Add fractional degrees to whole degrees to produce the final result   ###
### 5. If it is a Southern latitude or a Western longitude coordinate        ###
###    negate the result.                                                    ###
###                                                                          ###
### This script/demo was written by ERGO/Euro-AEGIS as a resource for the    ###
### partners in the EDENext FP7 project: Biology and control of vector-borne ###
### infections in Europe.                                                    ###
###                                                                          ###
### It was ammended from the original deg_to_dms script written by           ###
### Daniela Cianci (University of Utrecht)                                   ###
###                                                                          ###
### It can be freely distributed and re-used by all but please credit        ###
### Daniela Cianci and http://www.edenextdata.com                            ###
###                                                                          ###
### Author: Neil Alexander                                                   ###
### Contact: neil.alexander@zoo.ox.ac.uk                                     ###
### website: http://www.edenextdata.com                                      ###
###                                                                          ###
################################################################################
###                                                                          ###
###                                                                          ###


### Function to multiply out decimal degrees to DMS format
### Then concatenate output
dms_to_deg<- function (dms){
               splitdms <- strsplit(as.character(dms), ":")
               deg <- as.numeric(substring(splitdms[[1]][1],2,4))
               sec <- as.numeric(splitdms[[1]][3])+(as.numeric(splitdms[[1]][2])*60)
               secasdeg <- sec/3600
               dd <- deg + secasdeg
               return (dd)
              }

#### End Function ###

### Import coordinates in decimal degrees
### Tab delimited file two columns "latitude" and "longitude"
### Demo uses edeninstitutions.txt found in zipfile
coord<-read.table("edeninstitutions_dms.txt",sep="\t",header=T)

### Define matrix for results:
output<-matrix(NA,nrow(coord),2)
colnames(output)<-c("dd.lat","dd.lon")

### For each line in input check +/- value then add  N/E/S/W specifier
### Then apply function using value modulus if it is a negative value
for (i in 1: nrow(coord)){
  ### Ensure Input non-negative and define Direction Prefix:
  if (substr(coord$dms.lat[i],1,1) == "N") {
     dms_to_deg(coord$dms.lat[i]) ->output[i,1]
   }
  else {
     0 - dms_to_deg(coord$dms.lat[i]) ->output[i,1]
   }

  if (substr(coord$dms.lon[i],1,1) == "E") {
     dms_to_deg(coord$dms.lon[i]) ->output[i,2]
   }
  else {
     0 - dms_to_deg(coord$dms.lon[i])->output[i,2]
   }
}

### Save output to Tab delimited file output.txt
write.table(output, file="edeninstitutions_dd.txt", row.name=F, col.name=T, sep="\t")
